#!/usr/bin/env python
# encoding: utf-8
"""
Documentation for panstamps can be found here: http://panstamps.readthedocs.org

Usage:
    panstamps [options] [--width=<arcminWidth>] [--filters=<filterSet>] [--settings=<pathToSettingsFile>] [--downloadFolder=<path>] (warp|stack) <ra> <dec> [<mjdStart> <mjdEnd>]
    panstamps [options] --closest=<beforeAfter> [--width=<arcminWidth>] [--filters=<filterSet>] [--settings=<pathToSettingsFile>] [--downloadFolder=<path>] <ra> <dec> <mjd>

    -h, --help                              show this help message
    -f, --fits                              download fits (default on)
    -F, --nofits                            don't download fits (default off)
    -j, --jpeg                              download jepg (default off)
    -J, --nojpeg                            don't download jepg (default on)
    -c, --color                             download color jepg (default off)
    -C, --nocolor                           don't download color jepg (default on)
    -a, --annotate                          annotate jpeg (default true)
    -A, --noannotate                        don't annotate jpeg (default false)
    -t, --transient                         add a small red circle at transient location (default false)
    -T, --notransient                       don't add a small red circle at transient location (default true)
    -g, --greyscale                         convert jpeg to greyscale (default false)
    -G, --nogreyscale                       don't convert jpeg to greyscale (default true)
    -i, --invert                            invert jpeg colors (default false)
    -I, --noinvert                          don't invert jpeg colors (default true)
    --width=<arcminWidth>                   width of image in arcsec (default 1)
    --filters=<filterSet>                   filter set to download and use for color image (default gri)
    --downloadFolder=<path>                 path to the download folder, relative or absolute (folder created where command is run if not set)
    --settings=<pathToSettingsFile>         the settings file    
    --closest=<beforeAfter>                 return the warp closest in time to the given mjd. If you want to set a strict time window then pass in a positive or negative time in sec (before | after | secs)

    ra                                      right-ascension in sexagesimal or decimal degrees
    dec                                     declination in sexagesimal or decimal degrees
    mjdStart                                the start of the time-window within which to select images
    mjdEnd                                  the end of the time-window within which to select images
    mjd                                     report the warp closest in time to this mjd
    warp                                    select the warp images to download from STScI (IP of the machine running panstamps must be white=listed by MAST to allow for warp access)
    stack                                   select the stack images to download from STScI 
"""
import sys
import os
from os.path import expanduser
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
from docopt import docopt
from fundamentals import tools, times
from subprocess import Popen, PIPE, STDOUT
from panstamps.downloader import downloader
from panstamps.image import image


def tab_complete(text, state):
    return (glob.glob(text + '*') + [None])[state]


def main(arguments=None):
    """
    *The main function used when `cl_utils.py` is run as a single script from the cl, or when installed as a cl command*
    """
    # setup the command-line util settings
    su = tools(
        arguments=arguments,
        docString=__doc__,
        logLevel="WARNING",
        options_first=True,
        projectName="panstamps",
        defaultSettingsFile=True
    )
    arguments, settings, log, dbConn = su.setup()

    # tab completion for raw_input
    readline.set_completer_delims(' \t\n;')
    readline.parse_and_bind("tab: complete")
    readline.set_completer(tab_complete)

    # UNPACK REMAINING CL ARGUMENTS USING `EXEC` TO SETUP THE VARIABLE NAMES
    # AUTOMATICALLY
    a = {}
    for arg, val in list(arguments.items()):
        if arg[0] == "-":
            varname = arg.replace("-", "") + "Flag"
        else:
            varname = arg.replace("<", "").replace(">", "")
        a[varname] = val
        if arg == "--dbConn":
            dbConn = val
            a["dbConn"] = val
        log.debug('%s = %s' % (varname, val,))

    ## START LOGGING ##
    startTime = times.get_now_sql_datetime()
    log.info(
        '--- STARTING TO RUN THE cl_utils.py AT %s' %
        (startTime,))

    # set options interactively if user requests
    if "interactiveFlag" in a and a["interactiveFlag"]:

        # load previous settings
        moduleDirectory = os.path.dirname(__file__) + "/resources"
        pathToPickleFile = "%(moduleDirectory)s/previousSettings.p" % locals()
        try:
            with open(pathToPickleFile):
                pass
            previousSettingsExist = True
        except:
            previousSettingsExist = False
        previousSettings = {}
        if previousSettingsExist:
            previousSettings = pickle.load(open(pathToPickleFile, "rb"))

        # x-raw-input
        # x-boolean-raw-input
        # x-raw-input-with-default-value-from-previous-settings

        # save the most recently used requests
        pickleMeObjects = []
        pickleMe = {}
        theseLocals = locals()
        for k in pickleMeObjects:
            pickleMe[k] = theseLocals[k]
        pickle.dump(pickleMe, open(pathToPickleFile, "wb"))

    if "init" in a and a["init"]:

        home = expanduser("~")
        filepath = home + "/.config/panstamps/panstamps.yaml"
        try:
            cmd = """open %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass
        try:
            cmd = """start %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass
        return

    fitsFlag = a["fitsFlag"]
    nofitsFlag = a["nofitsFlag"]
    jpegFlag = a["jpegFlag"]
    nojpegFlag = a["nojpegFlag"]
    colorFlag = a["colorFlag"]
    nocolorFlag = a["nocolorFlag"]
    annotateFlag = a["annotateFlag"]
    noannotateFlag = a["noannotateFlag"]
    transientFlag = a["transientFlag"]
    notransientFlag = a["notransientFlag"]
    greyscaleFlag = a["greyscaleFlag"]
    nogreyscaleFlag = a["nogreyscaleFlag"]
    invertFlag = a["invertFlag"]
    noinvertFlag = a["noinvertFlag"]
    widthFlag = a["widthFlag"]
    filtersFlag = a["filtersFlag"]
    downloadFolderFlag = a["downloadFolderFlag"]
    closestFlag = a["closestFlag"]
    ra = a["ra"]
    dec = a["dec"]
    mjdStart = a["mjdStart"]
    mjdEnd = a["mjdEnd"]
    mjd = a["mjd"]
    stack = a["stack"]
    warp = a["warp"]

    # CALL FUNCTIONS/OBJECTS
    if ra:
        try:
            ra = float(ra)
        except:
            if ":" not in ra:
                log.error(
                    "ERROR: ra must be in decimal degree or sexagesimal format")
                return

    if dec:
        try:
            dec = float(dec)
        except:
            if ":" not in dec:
                log.error(
                    "ERROR: dec must be in decimal degree or sexagesimal format")
                return

    # BUILD KEYWORD DICT
    kwargs = {}
    kwargs["log"] = log
    kwargs["settings"] = settings
    kwargs["ra"] = ra
    kwargs["dec"] = dec

    # FITS OPTIONS
    kwargs["fits"] = True  # DEFAULT
    if fitsFlag == False and nofitsFlag == True:
        kwargs["fits"] = False

    # JPEG OPTIONS
    kwargs["jpeg"] = False  # DEFAULT
    if jpegFlag == True and nojpegFlag == False:
        kwargs["jpeg"] = True

    # COLOR JPEG OPTIONS
    kwargs["color"] = False  # DEFAULT
    if colorFlag == True and nocolorFlag == False:
        kwargs["color"] = True

    # WIDTH OPTION
    kwargs["arcsecSize"] = 60
    if widthFlag:
        kwargs["arcsecSize"] = float(widthFlag) * 60.

    # CHOOSE A FILTERSET
    kwargs["filterSet"] = 'gri'
    if filtersFlag:
        kwargs["filterSet"] = filtersFlag

    for i in kwargs["filterSet"]:
        if i not in "grizy":
            log.error(
                "ERROR: the requested filter must be in the grizy filter set")
            return

    # WHICH IMAGE TYPE TO DOWNLOAD
    if stack:
        kwargs["imageType"] = "stack"
    if warp:
        kwargs["imageType"] = "warp"
    if closestFlag:
        kwargs["imageType"] = "warp"

    # MJD WINDOW
    kwargs["mjdStart"] = mjdStart
    kwargs["mjdEnd"] = mjdEnd
    kwargs["window"] = False

    try:
        kwargs["window"] = int(closestFlag)
    except:
        pass

    if not kwargs["window"]:
        if mjd and closestFlag == "before":
            kwargs["mjdEnd"] = mjd
        elif mjd and closestFlag == "after":
            kwargs["mjdStart"] = mjd
    else:
        if mjd and kwargs["window"] < 0:
            kwargs["mjdEnd"] = mjd
        elif mjd and kwargs["window"] > 0:
            kwargs["mjdStart"] = mjd

    # DOWNLOAD LOCATION
    if downloadFolderFlag:
        home = expanduser("~")
        downloadFolderFlag = downloadFolderFlag.replace("~", home)
    kwargs["downloadDirectory"] = downloadFolderFlag

    # xt-kwarg_key_and_value

    # DOWNLOAD THE IMAGES
    images = downloader(**kwargs)
    fitsPaths, jpegPaths, colorPath = images.get()
    jpegPaths += colorPath

    # POST-DOWNLOAD PROCESS IMAGES
    kwargs = {}
    kwargs["log"] = log
    kwargs["settings"] = settings
    # WIDTH OPTION
    kwargs["arcsecSize"] = 60
    if widthFlag:
        kwargs["arcsecSize"] = float(widthFlag) * 60.

    # ANNOTATE JPEG OPTIONS
    kwargs["crosshairs"] = True  # DEFAULT
    kwargs["scale"] = True
    if annotateFlag == False and noannotateFlag == True:
        kwargs["crosshairs"] = False  # DEFAULT
        kwargs["scale"] = False

    # INVERT OPTIONS
    kwargs["invert"] = False  # DEFAULT
    if invertFlag == True and noinvertFlag == False:
        kwargs["invert"] = True

    # GREYSCALE OPTIONS
    kwargs["greyscale"] = False  # DEFAULT
    if greyscaleFlag == True and nogreyscaleFlag == False:
        kwargs["greyscale"] = True

    # TRANSIENT DOT OPTIONS
    kwargs["transient"] = False  # DEFAULT
    if transientFlag == True and notransientFlag == False:
        kwargs["transient"] = True

    for j in jpegPaths:
        kwargs["imagePath"] = j

        # kwargs["transient"] = False

        # kwargs["invert"] = False
        # kwargs["greyscale"] = False
        oneImage = image(**kwargs)
        oneImage.get()

    if "dbConn" in locals() and dbConn:
        dbConn.commit()
        dbConn.close()
    ## FINISH LOGGING ##
    endTime = times.get_now_sql_datetime()
    runningTime = times.calculate_time_difference(startTime, endTime)
    log.info('-- FINISHED ATTEMPT TO RUN THE cl_utils.py AT %s (RUNTIME: %s) --' %
             (endTime, runningTime, ))

    return

if __name__ == '__main__':
    main()



from __future__ import absolute_import
from . import cl_utils
from .__version__ import __version__
from . import utKit
from . import cl_utils
from .downloader import downloader
from .image import image



"""
*Unit testing tools*
"""
from fundamentals import utKit

# OVERRIDES


class utKit(utKit):
    """
    *Override dryx utKit*
    """
    # Variable Data Atrributes

    # Override Variable Data Atrributes
    # Initialisation
    def __init__(
            self,
            moduleDirectory,
            dbConn=False
    ):
        self.moduleDirectory = moduleDirectory
        # x-self-arg-tmpx

        # SETUP PATHS TO COMMON DIRECTORIES FOR TEST DATA
        self.pathToInputDir = moduleDirectory + "/input/"
        self.pathToOutputDir = moduleDirectory + "/output/"

        # SETUP LOGGING
        self.loggerConfig = """
        version: 1
        formatters:
            file_style:
                format: '* %(asctime)s - %(name)s - %(levelname)s (%(filename)s > %(funcName)s > %(lineno)d) - %(message)s  '
                datefmt: '%Y/%m/%d %H:%M:%S'
            console_style:
                format: '* %(asctime)s - %(levelname)s: %(filename)s:%(funcName)s:%(lineno)d > %(message)s'
                datefmt: '%H:%M:%S'
            html_style:
                format: '<div id="row" class="%(levelname)s"><span class="date">%(asctime)s</span>   <span class="label">file:</span><span class="filename">%(filename)s</span>   <span class="label">method:</span><span class="funcName">%(funcName)s</span>   <span class="label">line#:</span><span class="lineno">%(lineno)d</span> <span class="pathname">%(pathname)s</span>  <div class="right"><span class="message">%(message)s</span><span class="levelname">%(levelname)s</span></div></div>'
                datefmt: '%Y-%m-%d <span class= "time">%H:%M <span class= "seconds">%Ss</span></span>'
        handlers:
            console:
                class: logging.StreamHandler
                level: DEBUG
                formatter: console_style
                stream: ext://sys.stdout
        root:
            level: DEBUG
            handlers: [console]"""

        # Override Variable Data Atrributes
        self.dbConfig = False
        if dbConn:
            self.dbConfig = """
             version: 1
             db: dryx_unit_testing
             host: localhost
             user: unittesting
             password: utpass
             """

        return

    def get_project_root(self):
        """
        *Get the root of the `python` package - useful for getting files in the root directory of a project*

        **Return**

        - ``rootPath`` -- the root path of a project

        """
        import os
        rootPath = os.path.dirname(__file__)

        return rootPath

    def refresh_database(self):
        """
        *Refresh the unit test database*
        """
        from fundamentals.mysql import directory_script_runner
        from fundamentals import tools
        packageDirectory = self.get_project_root()
        su = tools(
            arguments={"settingsFile": packageDirectory +
                       "/test_settings.yaml"},
            docString=__doc__,
            logLevel="DEBUG",
            options_first=False,
            projectName=None,
            defaultSettingsFile=False
        )
        arguments, settings, log, dbConn = su.setup()
        directory_script_runner(
            log=log,
            pathToScriptDirectory=packageDirectory + "/tests/input",
            dbConn=dbConn,
            successRule=None,
            failureRule=None
        )



#!/usr/local/bin/python
# encoding: utf-8
"""
*Add crosshairs, logo, scale, orientation and fake transients to pre-downloaded PS1 images*

:Author:
    David Young
"""
from __future__ import division
from builtins import object
from past.utils import old_div
import sys
import os
os.environ['TERM'] = 'vt100'
from fundamentals import tools

class image(object):
    """
    *The worker class for the image module*

    **Key Arguments**

    - ``log`` -- logger
    - ``settings`` -- the settings dictionary
    - ``imagePath`` -- path to the image to manipulate
    - ``arcsecSize`` -- the size of the image stamps to download (1 arcsec == 4 pixels).
    - ``crosshairs`` -- add crosshairs to the image?. Default *True*
    - ``transient`` -- add a small transient marker at the centre of the image. Default *False*
    - ``scale`` -- add scale bar and orientation indicator to the image. Default *True*
    - ``invert`` -- invert the colours of the image. Default *False*
    - ``greyscale`` -- convert the image to greyscale. Default *False*
    - ``colorImage`` -- is the input image a color image, Default **False*. Note, also assumes a color image if 'color' in filename
    

    **Usage**

    ```python
    from panstamps.image import image
    myimage = image(
        log=log,
        settings=False,
        imagePath="70.60271m21.72433/color_igr_ra70.602710_dec-21.724330_arcsec120_skycell0812.050.jpeg",
        arcsecSize=120,
        crosshairs=True,
        transient=False,
        scale=True,
        invert=False,
        greyscale=False,
        colorImage=True
    ).get()
    ```

    Here's the resulting image from this code:

    .. image:: https://i.imgur.com/TXX2BS0.png
    
    """
    # Initialisation

    def __init__(
            self,
            log,
            imagePath,
            arcsecSize,
            settings=False,
            crosshairs=True,
            transient=False,
            scale=True,
            invert=False,
            greyscale=False,
            colorImage=False
    ):
        self.log = log
        log.debug("instansiating a new 'image' object")
        self.settings = settings
        self.imagePath = imagePath
        self.crosshairs = crosshairs
        self.transient = transient
        self.scale = scale
        self.invert = invert
        self.greyscale = greyscale
        self.arcsecSize = arcsecSize
        self.colorImage = colorImage
        # xt-self-arg-tmpx

        return None

    # Method Attributes
    def get(self):
        """
        *annotate the PS1 image*

        **Return**

        - ``image`` -- a PIL image object
        
        """
        self.log.debug('starting the ``get`` method')

        from PIL import Image, ImageDraw, ImageChops, ImageFont

        im = Image.open(self.imagePath)
        im = im.convert("RGB")

        # DETERMINE THE SIZE OF THE IMAGE
        imWidth, imHeight = im.size

        # THE CROSS HAIRS SHOULD BE 1/6 THE LENGTH OF THE SMALLEST DIMENSON
        chLen = int(old_div(min(imWidth, imHeight), 6))

        # THE GAP IN THE CENTRE SHOULD BE 1/60 OF THE LENGTH OF THE SMALLEST
        # DIMENSON
        gapLen = int(old_div(min(imWidth, imHeight), 60))

        # LINE WIDTH SHOULD BE EASILY VIEWABLE AT ALL SIZES - 0.2% OF THE WIDTH SEEMS GOOD
        # SEEMS FINE
        lineWidth = int(old_div(max(imWidth, imHeight), 350))

        lines = []
        l = (old_div(imWidth, 2) - gapLen - chLen, old_div(imHeight,
             2), old_div(imWidth, 2) - gapLen, old_div(imHeight, 2))
        lines.append(l)
        l = (old_div(imWidth, 2) + gapLen, old_div(imHeight,
             2), old_div(imWidth, 2) + gapLen + chLen, old_div(imHeight, 2))
        lines.append(l)
        l = (old_div(imWidth, 2), old_div(imHeight,
             2) - gapLen - chLen, old_div(imWidth, 2), old_div(imHeight, 2) - gapLen)
        lines.append(l)
        l = (old_div(imWidth, 2), old_div(imHeight,
             2) + gapLen, old_div(imWidth, 2), old_div(imHeight, 2) + gapLen + chLen)
        lines.append(l)

        if self.invert:
            im = ImageChops.invert(im)

        if self.greyscale:
            im = im.convert("L")
            im = im.convert("RGBA")

        # DETERMINE THE BEST COLOR FOR LINES
        # tmp = im.resize((1, 1), resample=3)
        # tmp = ImageChops.invert(tmp)
        # bestColor = tmp.getcolors()[0][1]

        if self.invert:
            bestColor = "#dc322f"
        elif not self.colorImage and 'color' not in self.imagePath.split("/")[-1]:
            bestColor = "#3c96ed"
        elif self.greyscale:
            bestColor = "#3c96ed"
        elif self.colorImage or 'color' in self.imagePath.split("/")[-1] and not self.invert:
            bestColor = "#7eb70a"
        else:
            bestColor = "#dc322f"

        # GENERATE THE DRAW OBJECT AND DRAW THE CROSSHAIRS
        draw = ImageDraw.Draw(im)
        if self.crosshairs:
            for l in lines:
                draw.line(l, fill=bestColor, width=lineWidth)

        from PIL import Image, ImageDraw, ImageChops, ImageFont

        # DRAW A SCALEBAR ON THE IMAGE
        physicalWidth = int(self.arcsecSize)
        startRatio = 0.3
        sbPhysicalWidth = physicalWidth * startRatio
        sbPixelWidth = imWidth * startRatio

        # IF ON THE DEGREE SCALE
        if sbPhysicalWidth > 3600:
            divider = 3600
            unit = "degree"
        # IF ON THE ARCMIN SCALE
        elif sbPhysicalWidth > 60:
            divider = 60.
            unit = "arcmin"
        # IF ON THE ARCSEC SCALE
        else:
            divider = 1.
            unit = "arcsec"

        # FIND THE WIDTH OF THE BAR TO THE NEAREST WHOLE NUMBER ON GIVEN SCALE
        tmpWidth = old_div(sbPhysicalWidth, divider)
        self.log.debug("physicalWidth = %(physicalWidth)s" % locals())
        self.log.debug("sbPhysicalWidth = %(sbPhysicalWidth)s" % locals())
        displayPhysicalWidth = old_div(sbPhysicalWidth, divider)
        self.log.debug(
            "displayPhysicalWidth = %(displayPhysicalWidth)s" % locals())
        displayPhysicalWidth = int(old_div(sbPhysicalWidth, divider))
        self.log.debug(
            "displayPhysicalWidth = %(displayPhysicalWidth)s" % locals())
        ratio = old_div(displayPhysicalWidth, tmpWidth)
        sbPhysicalWidth = int(sbPhysicalWidth * ratio)
        sbPixelWidth = int(sbPixelWidth * ratio)

        # DRAW THE SCALEBAR
        l = (old_div(imWidth, 20), imHeight - old_div(imHeight, 20),
             old_div(imWidth, 20) + sbPixelWidth, imHeight - old_div(imHeight, 20))
        if self.scale:
            draw.line(l, fill=bestColor, width=lineWidth)

        # ADD SCALE TEXT
        text = """%(displayPhysicalWidth)s %(unit)s""" % locals()
        fontsize = int(old_div(imWidth, 30))
        moduleDirectory = os.path.dirname(__file__)
        font = ImageFont.truetype(
            moduleDirectory + "/../resources/fonts/source-sans-pro-regular.ttf", fontsize)
        if self.scale:
            draw.text((old_div(imWidth, 20), imHeight - old_div(imHeight, 20) - fontsize * 1.3), text, fill=bestColor,
                      font=font, anchor=None)

        # ADD ORIENTATION INDICATOR
        lines = []
        lineLength = int(old_div(min(imWidth, imHeight), 20))
        l = (imWidth - old_div(imWidth, 20) - lineLength, imHeight - old_div(imHeight, 20),
             imWidth - old_div(imWidth, 20), imHeight - old_div(imHeight, 20))
        lines.append(l)
        l = (imWidth - old_div(imWidth, 20), imHeight - old_div(imHeight, 20),
             imWidth - old_div(imWidth, 20), imHeight - old_div(imHeight, 20) - lineLength)
        lines.append(l)
        if self.scale:
            for l in lines:
                draw.line(l, fill=bestColor, width=lineWidth)
            # ADD SCALE TEXT
            draw.text((imWidth - old_div(imWidth, 20) - fontsize * 0.3, imHeight - old_div(imHeight, 20) - lineLength - fontsize * 1.3), "N", fill=bestColor,
                      font=font, anchor=None)
            draw.text((imWidth - old_div(imWidth, 20) - lineLength - fontsize * 0.8, imHeight - old_div(imHeight, 20) - fontsize * 0.6), "E", fill=bestColor,
                      font=font, anchor=None)
        del draw

        # ADD THE PS1 LOGO
        wmHeight = int(old_div(max(imWidth, imHeight), 20))
        moduleDirectory = os.path.dirname(__file__)
        imagePath = moduleDirectory + "/../resources/ps1.png"
        logo = Image.open(imagePath)
        (logoWidth, logoHeight) = logo.size
        wmWidth = int((wmHeight / float(logoHeight)) * logoWidth)
        logo = logo.resize((wmWidth, wmHeight), resample=3)

        # CREATE A TRANSPARENT IMAGE THE SIZE OF THE ORIGINAL IMAGE - PASTE THE
        # LOGO WHERE REQUIRED
        logoPH = Image.new("RGBA", (imWidth, imHeight), color=(0, 0, 0, 0))
        logoPH.paste(logo, box=(old_div(imHeight, 25), old_div(imHeight, 25)))

        # NOW TONE DOWN THE OPACITY
        trans = Image.new("RGBA", (imWidth, imHeight), color=(0, 0, 0, 0))
        logo = Image.blend(trans, logoPH, alpha=0.75)

        # ADD THE WATERMARK STAMP TO THE ORIGINAL IMAGE
        im = Image.alpha_composite(im.convert("RGBA"), logo)

        # ADD A TRANSIENT MARKER
        outercircle = 60
        if self.transient:
            draw = ImageDraw.Draw(im)
            xy1 = (old_div(imWidth, 2) - old_div(imWidth, 100), old_div(imWidth, 2) - old_div(imWidth, 100),
                   old_div(imWidth, 2) + old_div(imWidth, 100), old_div(imWidth, 2) + old_div(imWidth, 100))
            xy2 = (old_div(imWidth, 2) - old_div(imWidth, outercircle), old_div(imWidth, 2) - old_div(imWidth, outercircle),
                   old_div(imWidth, 2) + old_div(imWidth, outercircle), old_div(imWidth, 2) + old_div(imWidth, outercircle))
            xy3 = (old_div(imWidth, 2) - old_div(imWidth, (outercircle)) - 1, old_div(imWidth, 2) - old_div(imWidth, (outercircle)) - 1,
                   old_div(imWidth, 2) + old_div(imWidth, (outercircle)) + 1, old_div(imWidth, 2) + old_div(imWidth, (outercircle)) + 1)
            xy4 = (old_div(imWidth, 2) - old_div(imWidth, (outercircle)) - 2, old_div(imWidth, 2) - old_div(imWidth, (outercircle)) - 2,
                   old_div(imWidth, 2) + old_div(imWidth, (outercircle)) + 2, old_div(imWidth, 2) + old_div(imWidth, (outercircle)) + 2)
            draw.arc(xy2, 0, 360, fill="#b2141c")
            draw.arc(xy3, 0, 360, fill="#b2141c")
            draw.arc(xy4, 0, 360, fill="#b2141c")
            draw.pieslice(xy1, 0, 361, fill="#b2141c")

        im = im.convert("RGB")
        im.save(self.imagePath)

        self.log.debug('completed the ``get`` method')
        return image

    # xt-class-method
